import numpy as np
import matplotlib.pyplot as plt
import sys
# to run this type:
# python passiveChannelsToUTM.py npy myFileNameWithout.npyEnding
# or 
# python passiveChannelsToUTM.py txt myFileNameWithout.txtEnding
#
# depending on whether you want a numpy binary or a txt output 
# of channels, East UTM coordinates, and North UTM coordinates
# The txt file output has the format
# channel(i) \t east UTM(i) \t north UTM(i) \n 
# for each channel of interest.

# trios of channel number, East UTM, North UTM
GreenManholeStart = (14,573088.0,4142497.0)
PanamaManhole12 = (25,573000.58,4142531.35)
RobleManhole09Start = (48,572851.00,4142560.00)
RobleManhole09End = (49,572851.00,4142560.0)
ViaOrtegaAndPanamaManhole = (58,572871.35,4142626.52)
ViaOrtegaManhole07 = (70,572889.00,4142710.00)
ViaOrtegaManhole13 = (83,572920.19,4142817.00)
NWCornerOfAllenManhole = (96,572961.29,4142909.74)
StartNorthCoil = (100,572942.64,4142936.69)
EndNorthCoil = (107,572942.64,4142936.69)
PanamaAndCampus = (138,572694.11,4142985.43)
PanamaManhole30 = (155,572693.26,4142866.36)
PanamaCurveStart = (157,572695.88,4142872.24)
PanamaCurveEnd = (165,572736.10,4142875.45)
PineManhole19 = (167,572740.38,4142868.55)
EWViaOrtegaViaPueblo = (184,572922.4,4142826.52)
StartCoilByAllen = (203,573047.61,4142791.85)
EndCoilByAllen = (209,573047.61,4142791.85)
StartJogByHewlett = (225,573172.14,4142767.36)
EndJogByHewlett = (228,573172.14,4142767.36)
StartSequoiaJog = (240,573249.00,4142746.00)
TopSequoiaJog = (245,573258.00,4142770.00)
MooreManhole32 = (261,573221.45,4142639.24)
BikeRacksNorthOfMitchell = (269,573205.43,4142563.1)
BetweenDurMitSki = (274,573173.9,4142544.07)
WestOfMitchell = (280,573161.0,4142501.0)
MitchellManhole1 = (283,573188.0,4142468.0)
MitchellManhole2 = (288,573188.0,4142468.0)
GreenManholeEnd = (302,573088.84,4142497.62)
refChEN = [GreenManholeStart,PanamaManhole12,RobleManhole09Start,RobleManhole09End,ViaOrtegaAndPanamaManhole,ViaOrtegaManhole07,ViaOrtegaManhole13,NWCornerOfAllenManhole,StartNorthCoil,EndNorthCoil,PanamaAndCampus,PanamaManhole30,PanamaCurveStart,PanamaCurveEnd,PineManhole19,EWViaOrtegaViaPueblo,StartCoilByAllen,EndCoilByAllen,StartJogByHewlett,EndJogByHewlett,StartSequoiaJog,TopSequoiaJog,MooreManhole32,BikeRacksNorthOfMitchell,BetweenDurMitSki,WestOfMitchell,MitchellManhole1,MitchellManhole2,GreenManholeEnd]
firstCh = 14
lastCh = 302
channels = np.arange(firstCh,lastCh+1)
EastUTM = np.zeros(lastCh-firstCh+1)
NorthUTM = np.zeros(lastCh-firstCh+1)
# fill in first point
EastUTM[0] = (refChEN[0])[1]
NorthUTM[0] = (refChEN[0])[2]
prevPt = refChEN[0]
for pt in refChEN:
    if(pt[0] > firstCh):
        numberChs = pt[0]-prevPt[0]
        dE = (pt[1]-prevPt[1])/float(numberChs)
        dN = (pt[2]-prevPt[2])/float(numberChs)
        for Ch in range(prevPt[0]+1,pt[0]+1):
            EastUTM[Ch-firstCh] = prevPt[1]+dE*(Ch-prevPt[0])
            NorthUTM[Ch-firstCh] = prevPt[2]+dN*(Ch-prevPt[0])
    prevPt = pt

# plot geometry with marker color corresponding to channel number
dCh = 0.99/(lastCh-firstCh)
ChScaled = (channels-firstCh)*dCh
plt.scatter(EastUTM,NorthUTM,c=ChScaled,s=4,linewidth=0)
plt.xlabel('East UTM (m)')
plt.ylabel('West UTM (m)')
plt.title('UTM locations')
plt.savefig('fig/channelMapping.pdf')

# save the geometry information
if(sys.argv[1] == 'npy'):    
    np.savez(sys.argv[2],channels,EastUTM,NorthUTM)
elif(sys.argv[1] == 'txt'):
    outfile = (sys.argv[2])+'.txt'
    out = open(outfile,'w')
    for i in range(firstCh,lastCh+1):
        out.write(str(i)+'\t'+str(EastUTM[i-firstCh])+'\t'+str(NorthUTM[i-firstCh])+'\n')
    out.close()
